#!/usr/bin/env python

import random
import math
from components import *

class Setup:
    def __init__(self, application, communication, resource, 
                 field_sizes=(100, 60, None), sensor=None):
        self.app = application
        self.com = communication 
        self.res = resource
        self.field_sizes = field_sizes
        self.sensor = sensor
        self.nodes = None
        self.rng_config = None
        self.dummy_net = False

class TopCamera(Setup):
    def __init__(self, application, 
                 com=Communication(Radio(), MacTMAC(101, 10, 5000), Routing()),
                 resource=Resource(), fov_bb=None, sensor_type=SensorCamera, 
                 field_sizes=(100, 60, None)):
        Setup.__init__(self, application, com, resource, field_sizes,
                      (sensor_type(fov_bb=fov_bb) if fov_bb else sensor_type()))

    def create_network(self, 
                     proc_factory=PhysicalProcessFactory('WiseMovingTarget', 1),
                     w_channel=WirelessChannel(), time_limit=None):
        if not self.nodes:
            raise BaseException('Node deployment not yet created.')
        w_channel.dummy = self.dummy_net
        sn = SensorNetwork(w_channel, proc_factory, self.nodes, 
                           time_limit=time_limit, rng_config=self.rng_config)
        sn.set_same_flag('app', True)
        sn.set_same_flag('sens', True)
        sn.set_same_flag('res', True)
        sn.set_same_flag('com', True)
        return sn
        

class TopCameraRandom(TopCamera):
    def uniform(self, n_nodes):
        cam_w, cam_h =  self.sensor.get_fov_bb() 
        field_x = self.field_sizes[0]
        field_y = self.field_sizes[1]
        random.seed(0) # TODO: change this?
        pos_x = []
        pos_y = []
        for i in range(n_nodes):
            pos_x.append(random.uniform(0, field_x))
            pos_y.append(random.uniform(0, field_y))
            #off_x = (cam_w / 4) - 5
            #off_y = (cam_h / 4) - 5
            off_x = 10
            off_y = 10
        self.nodes = []
        first_node = True
        for n, (x, y) in enumerate(map(None, pos_x, pos_y)): 
            if first_node:
                node = Node(n, x, y, app=self.app, com=self.com,
                            sensor=self.sensor, res=self.res)
                first_node = False
            else:
                node = Node(n, x, y)
            self.nodes.append(node)

    def uniform2(self, n_nodes):
        field_x = self.field_sizes[0]
        field_y = self.field_sizes[1]
        cam_w, cam_h =  self.sensor.get_fov_bb() 
        n_x = int(math.ceil(float(field_x) / float(cam_w)))
        n_y = int(math.ceil(float(field_y) / float(cam_h)))
        pos = []
        cnt = 0
        y = cam_h / 2
        for j in range(n_y):
            x = cam_w / 2
            for i in range(n_x):
                if cnt >= n_nodes:
                    break
                pos.append((x, y))
                x = x + cam_w
                cnt += 1
            if cnt >= n_nodes:
                break
            y = y + cam_h
        y = cam_h / 4
        for j in range(n_y):
            x = cam_w / 4
            for i in range(n_x):
                if cnt >= n_nodes:
                    break
                pos.append((x, y))
                x = x + cam_w
                cnt += 1
            if cnt >= n_nodes:
                break
            y = y + cam_h
        random.seed(0) # TODO: change this?
        pos_x = []
        pos_y = []
        for i in range(n_nodes - cnt):
            pos_x.append(random.uniform(0, field_x))
            pos_y.append(random.uniform(0, field_y))
        pos = pos + map(None, pos_x, pos_y) 
        self.nodes = []
        first_node = True
        for n, (x, y) in enumerate(pos): 
            if first_node:
                node = Node(n, x, y, app=self.app, com=self.com,
                            sensor=self.sensor, res=self.res)
                first_node = False
            else:
                node = Node(n, x, y)
            self.nodes.append(node)
            
class TopCameraRandomUniform(TopCameraRandom):
    def setup(self, n_nodes):
        self.uniform(n_nodes)

